@@ -8,9 +8,8 @@ from ..diagonal import Diagonal
 from ..kronecker import Kronecker
 from ..lowrank import LowRank
 from ..matrix import AbstractMatrix, Dense, structured
-from ..shape import get_shape
 from ..triangular import LowerTriangular, UpperTriangular
-from ..util import ToDenseWarning, redirect
+from ..util import redirect, ToDenseWarning
 from ..woodbury import Woodbury
 
 __all__ = []
@@ -20,8 +19,8 @@ def _tr(a, do):
     return B.transpose(a) if do else a
 
 
-def _shape_tr(a, tr_a):
-    ar, ac = B.shape(a)
+def _shape_matrix_tr(a, tr_a):
+    ar, ac = B.shape_matrix(a)
     if tr_a:
         return ac, ar
     else:
@@ -29,9 +28,10 @@ def _shape_tr(a, tr_a):
 
 
 def _assert_composable(a, b, tr_a: bool = False, tr_b: bool = False):
-    s1, s2 = get_shape(a, b)
-    s1 = _tr(s1, tr_a)
-    s2 = _tr(s2, tr_b)
+    s1 = B.shape_matrix(a)
+    s2 = B.shape_matrix(b)
+    s1 = (s1[1], s1[0]) if tr_a else s1
+    s2 = (s2[1], s2[0]) if tr_b else s2
     assert (
         s1[1] == s2[0]
     ), f"Objects {a} and {b} are asserted to be composable, but they are not."
@@ -43,17 +43,17 @@ def _assert_composable(a, b, tr_a: bool = False, tr_b: bool = False):
 @B.dispatch(precedence=proven())
 def matmul(a: AbstractMatrix, b: Zero, tr_a: bool = False, tr_b: bool = False):
     _assert_composable(a, b, tr_a=tr_a, tr_b=tr_b)
-    ar, _ = _shape_tr(a, tr_a)
-    _, bc = _shape_tr(b, tr_b)
-    return Zero(b.dtype, ar, bc)
+    ar, _ = _shape_matrix_tr(a, tr_a)
+    _, bc = _shape_matrix_tr(b, tr_b)
+    return Zero(b.dtype, *B.shape_batch_broadcast(a, b), ar, bc)
 
 
 @B.dispatch(precedence=proven())
 def matmul(a: Zero, b: AbstractMatrix, tr_a: bool = False, tr_b: bool = False):
     _assert_composable(a, b, tr_a=tr_a, tr_b=tr_b)
-    ar, _ = _shape_tr(a, tr_a)
-    _, bc = _shape_tr(b, tr_b)
-    return Zero(b.dtype, ar, bc)
+    ar, _ = _shape_matrix_tr(a, tr_a)
+    _, bc = _shape_matrix_tr(b, tr_b)
+    return Zero(a.dtype, *B.shape_batch_broadcast(a, b), ar, bc)
 
 
 # Multiple multiplication:
@@ -61,9 +61,9 @@ def matmul(a: Zero, b: AbstractMatrix, tr_a: bool = False, tr_b: bool = False):
 
 @B.dispatch
 def matmul(a, b, c, tr_a: bool = False, tr_b: bool = False, tr_c: bool = False):
-    ar, ac = _shape_tr(a, tr_a)
-    br, bc = _shape_tr(b, tr_b)
-    cr, cc = _shape_tr(c, tr_c)
+    ar, ac = _shape_matrix_tr(a, tr_a)
+    br, bc = _shape_matrix_tr(b, tr_b)
+    cr, cc = _shape_matrix_tr(c, tr_c)
 
     cost_ab_first = ar * ac * bc + ar * bc * cc
     cost_bc_first = br * bc * cc + ar * br * cc
@@ -94,13 +94,13 @@ def matmul(a: Diagonal, b: Diagonal, tr_a: bool = False, tr_b: bool = False):
 @B.dispatch
 def matmul(a: Diagonal, b: AbstractMatrix, tr_a: bool = False, tr_b: bool = False):
     _assert_composable(a, b, tr_a=tr_a, tr_b=tr_b)
-    return Dense(B.expand_dims(a.diag, axis=1)) * _tr(b, tr_b)
+    return Dense(B.expand_dims(a.diag, axis=-1)) * _tr(b, tr_b)
 
 
 @B.dispatch
 def matmul(a: AbstractMatrix, b: Diagonal, tr_a: bool = False, tr_b: bool = False):
     _assert_composable(a, b, tr_a=tr_a, tr_b=tr_b)
-    return _tr(a, tr_a) * Dense(B.expand_dims(b.diag, axis=0))
+    return _tr(a, tr_a) * Dense(B.expand_dims(b.diag, axis=-2))
 
 
 # Constant
@@ -121,7 +121,7 @@ def matmul(a: Constant, b: AbstractMatrix, tr_a: bool = False, tr_b: bool = Fals
     b = _tr(b, tr_b)
     return LowRank(
         B.ones(B.dtype(a), a.rows, 1),
-        B.expand_dims(B.sum(b, axis=0), axis=1),
+        B.expand_dims(B.sum(b, axis=-2), axis=-1),
         B.fill_diag(a.const, 1),
     )
 
@@ -132,7 +132,7 @@ def matmul(a: AbstractMatrix, b: Constant, tr_a: bool = False, tr_b: bool = Fals
     a = _tr(a, tr_a)
     b = _tr(b, tr_b)
     return LowRank(
-        B.expand_dims(B.sum(a, axis=1), axis=1),
+        B.expand_dims(B.sum(a, axis=-1), axis=-1),
         B.ones(B.dtype(b), b.cols, 1),
         B.fill_diag(b.const, 1),
     )
@@ -244,10 +244,9 @@ def matmul(a: LowRank, b: LowRank, tr_a: bool = False, tr_b: bool = False):
     middle = B.matmul(a.right, b.left, tr_a=True)
     middle = B.matmul(a.middle, middle, b.middle)
     # Let `middle` be of type `Diagonal` if possible.
-    if B.shape(middle) == (1, 1):
-        return LowRank(a.left, b.right, Diagonal(middle[0]))
-    else:
-        return LowRank(a.left, b.right, middle)
+    if B.shape_matrix(middle) == (1, 1):
+        middle = Diagonal(B.expand_dims(middle[..., 0, 0], axis=-1))
+    return LowRank(a.left, b.right, middle)
 
 
 @B.dispatch
@@ -324,20 +323,21 @@ def matmul(a: Kronecker, b: Kronecker, tr_a: bool = False, tr_b: bool = False):
 
 
 def _reshape_cols(a, *indices):
-    return B.transpose(B.reshape(B.transpose(a), *reversed(indices)))
+    return B.transpose(B.reshape(B.transpose(a), *B.shape_batch(a), *reversed(indices)))
 
 
-_reshape_rows = B.reshape
+def _reshape_rows(a, *indices):
+    return B.reshape(a, *B.shape_batch(a), *indices)
 
 
 def _kron_id_a_b(a, b):
-    reshaped = _reshape_cols(b, B.shape(a)[1], -1)
-    return _reshape_cols(B.matmul(a, reshaped), -1, B.shape(b)[1])
+    reshaped = _reshape_cols(b, B.shape(a, -1), -1)
+    return _reshape_cols(B.matmul(a, reshaped), -1, B.shape(b, -1))
 
 
 def _kron_a_id_b(a, b):
-    reshaped = _reshape_rows(b, B.shape(a)[1], -1)
-    return _reshape_rows(B.matmul(a, reshaped), -1, B.shape(b)[1])
+    reshaped = _reshape_rows(b, B.shape(a, -1), -1)
+    return _reshape_rows(B.matmul(a, reshaped), -1, B.shape(b, -1))
 
 
 @B.dispatch
@@ -349,16 +349,16 @@ def matmul(a: Kronecker, b: AbstractMatrix, tr_a: bool = False, tr_b: bool = Fal
     # Extract the factors of the product.
     left = a.left
     right = a.right
-    l_rows, l_cols = B.shape(left)
-    r_rows, r_cols = B.shape(right)
+    l_rows, l_cols = B.shape_matrix(left)
+    r_rows, r_cols = B.shape_matrix(right)
 
     cost_first_left = l_rows * l_cols * r_cols + r_rows * r_cols * l_rows
     cost_first_right = r_rows * r_cols * l_cols + l_rows * l_cols * r_rows
 
     if cost_first_left <= cost_first_right:
-        return _kron_id_a_b(right, _kron_a_id_b(left, b))
+        return convert(_kron_id_a_b(right, _kron_a_id_b(left, b)), AbstractMatrix)
     else:
-        return _kron_a_id_b(left, _kron_id_a_b(right, b))
+        return convert(_kron_a_id_b(left, _kron_id_a_b(right, b)), AbstractMatrix)
 
 
 @B.dispatch
